#include "osdsh.h"
#include "../config.h"

    settings_t settings;
    xosd *mainosd;

    pthread_t control_thread;
    pthread_t mixer_thread;
    pthread_t clock_thread; 
    pthread_t ppp_thread;
    pthread_t apm_thread;

    void *apm_watch(void *arg);
    void *control_sh(void *arg);
    void *mixer_watch(void *arg);
    void *connection_watch(void *arg);
    void *clock_display(void *arg);

    extern char clockformat[255];
    extern char mixer[PATH_MAX+1];
    extern char pppif[255];

/*basics, for each one of the "features"*/
    extern settings_t mixerset;
    extern settings_t clockset;
    extern settings_t pppset;
    extern settings_t apmset;
    extern xosd *mixerosd;
    extern xosd *clockosd;
    extern xosd *ppposd;
    extern xosd *apmosd;
    extern char connecting1[BUFSIZ];
    extern char connecting2[BUFSIZ];
    extern char connected1[BUFSIZ];
    extern char connected2[BUFSIZ];
    extern char disconnected[BUFSIZ];
    extern char connecttime[BUFSIZ];


/* ============================ control_options() =========================== */
void control_options(settings_t *setts, xosd *osd, char option[2], char argument[BUFSIZ], int refresh)
{
    /*this function reads the option to change on each display, changes it
    and saves the changes on the corresponding settings structure*/

    switch (option[0]) {
	case 'f':
	    strncpy(setts->font, argument, BUFSIZ);
	    if (refresh) 
		xosd_set_font(osd, argument);
    	break;
    	case 'c':
	    strncpy(setts->color, argument, BUFSIZ);
	    if (refresh)  
		xosd_set_colour(osd, argument);
	break;
	case 'd':
	    setts->delay = atoi(argument);
	    if (refresh)
		xosd_set_timeout(osd, atoi(argument));
	break;
	case 'b':
	    setts->yposition = XOSD_bottom;
	    if (refresh)
		xosd_set_pos(osd, setts->xposition, XOSD_bottom);
	break;
	case 't':
	    setts->yposition = XOSD_top;
	    if (refresh)
		xosd_set_pos(osd, setts->xposition, XOSD_top);
	break;
	case 'l':
	    setts->xposition = XOSD_left;
	    if (refresh)
		xosd_set_pos(osd, XOSD_left, setts->yposition);
	break;
	case 'r':
	    setts->xposition = XOSD_right;
	    if (refresh)
		xosd_set_pos(osd, XOSD_right, setts->yposition);
	break;
	case 'o':
	    setts->soffset = atoi(argument);
	    if (refresh)
		xosd_set_shadow_offset(osd, atoi(argument));
	break;
	case 'y':
	    setts->yoffset = atoi(argument);
	    if (refresh)
		xosd_set_offset(osd, setts->xoffset, atoi(argument));
	    break;
	case 'x':
	    setts->xoffset = atoi(argument);
	    if (refresh)
		xosd_set_offset(osd, atoi(argument), setts->yoffset);
	default:
	break;
    }
}

/*============================ display_stuff()=============================== */
void display_stuff(xosd *osd, char command[BUFSIZ], char arguments[BUFSIZ])
{
    char c;
    char arg_first[BUFSIZ];
    char arg_secound[BUFSIZ];
    int a,i;

    sprintf(arg_first, "");
    sprintf(arg_secound, "");

    /* separate the arguments in sub_arguments .... or not?*/
    i=0;
    a=0;
    while ((c=arguments[i])!=','&&c!='\n'&&c!='\0') {
	if (c=='\\') {
	    i++;
	    arg_first[a] = arguments[i];
	}
	else {
	    arg_first[a] = c;
	}
	a++;
	i++;
    }
    arg_first[a] = '\0';
    i++;
    a = 0;
    while((c=arguments[i])!='\n'&&c!='\0'&&c!=')') {
	if (c=='\\') {
	    i++;
	    arg_secound[a] = arguments[i];
	}
        else {
	    arg_secound[a] = c;
	}
	a++;
	i++;
    }
    arg_secound[a] = '\0';

    /*done with the arguments, now lets check what are we supposed to do*/
    /* strings, bars, sliders, display clock, exit, or change the look of
    one of our displays....*/ 
    if (strcmp(command, "\n")==0) 
	return;
    if (strcmp(command, "strn")==0) {
	xosd_display(osd, 0, XOSD_string, arg_first);
	xosd_display(osd, 1, XOSD_string, arg_secound);
    }
    else if (strcmp(command, "bars")==0) {
	xosd_display(osd, 0, XOSD_string, arg_first);
	xosd_display(osd, 1, XOSD_percentage, atoi(arg_secound));
     }
    else if (strcmp(command, "slid")==0) {
	xosd_display(osd, 0, XOSD_string, arg_first);
	xosd_display(osd, 1, XOSD_slider, atoi(arg_secound));
    }
    else if (strcmp(command, "dset")==0) {
	control_options(&settings, mainosd, arg_first, arg_secound,
			settings.display);
    }
    else if (strcmp(command, "smix")==0) {
	control_options(&mixerset, mixerosd, arg_first, arg_secound,
			mixerset.display);
    }
    else if (strcmp(command, "sclk")==0) {
	control_options(&clockset, clockosd, arg_first, arg_secound,
			clockset.display);
    }
    else if (strcmp(command, "sppp")==0) {
	control_options(&pppset, ppposd, arg_first, arg_secound,
			pppset.display);
    }
    else if (strcmp(command, "sapm")==0) {
	control_options(&apmset, apmosd, arg_first, arg_secound,
			apmset.display);
    }
    else if (strcmp(command, "clck")==0) {
	if (!clockset.display&&atoi(arg_first)) {
	    clockset.display = 1;
	    pthread_create(&clock_thread, NULL, clock_display, NULL);
	}
	else {
	    clockset.display = atoi(arg_first);
	}
    }
    else if (strcmp(command, "clkf")==0) {
	strcpy(clockformat ,arg_first);
    }
    else if (strcmp(command, "time")==0) {
	if (!clockset.display) {
	    clockset.display = 1;
	    pthread_create(&clock_thread, NULL, clock_display, NULL);
	}
	sleep(atoi(arg_first));
	clockset.display = 0;
    }
    else if (strcmp(command, "mixr")==0) {
	if (!mixerset.display&&atoi(arg_first)) {
	    mixerset.display = 1;
	    pthread_create(&mixer_thread, NULL, mixer_watch, NULL);
	}
	else {
	    mixerset.display = atoi(arg_first);
	}
    }
    else if (strcmp(command, "pppw")==0) {
	if (!pppset.display&&atoi(arg_first)) {
	    pppset.display = 1;
	    pthread_create(&ppp_thread, NULL, connection_watch, NULL);
	}
	else {
	pppset.display = atoi(arg_first);
	}
    }
    else if (strcmp(command, "pdev")==0) {
	sprintf(pppif, "%s", arg_first);
    }
    else if (strcmp(command, "apmw")==0) {
	if (!apmset.display&&atoi(arg_first)) {
	    apmset.display = 1;
	    pthread_create(&apm_thread, NULL, apm_watch, NULL);
	}
	else {
	apmset.display = atoi(arg_first);
	}
    }
    else if (strcmp(command, "apms")==0) {
	if (!apmset.display) {
	    apmset.display = 1;
	    pthread_create(&apm_thread, NULL, apm_watch, NULL);
	}
	sleep(atoi(arg_first));
	apmset.display = 0;
    }
    else if (strcmp(command, "exit")==0) {
	xosd_display(osd, 0, XOSD_string, "Goodbye");
	xosd_display(osd, 1, XOSD_string, "");
	sleep(1);
	xosd_uninit(mainosd);
	exit(0);
    }
    else if (strcmp(command, "sleep")==0) {
	sleep(atoi(arg_first));
    }
    else if (strcmp(command, "connecting")==0) {
	sprintf(connecting1, "%s", arg_first);
	sprintf(connecting2, "%s", arg_secound);
    }
    else if (strcmp(command, "connected")==0) {
	sprintf(connected1, "%s", arg_first);
	sprintf(connected2, "%s", arg_secound);
    }
    else if (strcmp(command, "disconnected")==0) {
	sprintf(disconnected, "%s", arg_first);
    }
    else if (strcmp(command, "showconntime")==0) {
	xosd_display(ppposd, 0, XOSD_string, connecttime);
	xosd_display(ppposd, 1, XOSD_string, "");
    }
}

/*================================control_sh() ============================= */
void *control_sh(void *arg)
{
    int a,i;
    char c;
    char command[BUFSIZ];
    char command_arg[BUFSIZ];

    char fifo_file[PATH_MAX +1];

    FILE *fp;
    char readbuf[BUFSIZ];

    sprintf(fifo_file, "%s.%d",OSD_FIFO_PATH, getuid());
    settings.display = 1;
    /* we open the osd*/
	mainosd =
	    xosd_init(settings.font, settings.color, settings.delay,
		settings.xposition, settings.yposition,
		settings.soffset, settings.xoffset, settings.yoffset);

    /* create the fifo file*/
	umask(0);
	mknod(fifo_file, S_IFIFO|0600, 0);
	    /*open and read the fifo*/
	while (1) {
		fp = fopen(fifo_file, "r");
		while(fgets(readbuf, BUFSIZ, fp)) {
		    /*ignore spaces*/
		    for (i=0; i<BUFSIZ&&((c=readbuf[i])==' '||c=='\t'); i++);
		    /*get the command*/
		    for (a=0; (c=readbuf[i])!='\n'&&c!='('; a++,i++)
			command[a] = readbuf[i];
		    command[a] = '\0';
		    /*get the command's arguments*/
		    i ++;
		    for(a=0; (c=readbuf[i])!='\0'&&c!='\n'&&c!=')';a++,i++) {
			if (c=='\\') {
			    i++;
			    command_arg[a] = readbuf[i];
		        }
			else {
			    command_arg[a]=c;
			}
		    }
		    command_arg[a] = '\0';
		    /*execute the command from the fifo and close it*/
		    display_stuff(mainosd, command, command_arg);
		    /*sprintf(readbuf, "");*/
		}
		fclose(fp);
	}
}
