#include "osdsh.h"
#include "../config.h"

    extern char mixer[PATH_MAX+1];
    int dev;
    settings_t mixerset;
    xosd *mixerosd;
    int vol[SOUND_MIXER_NRDEVICES];
    int old_vol[SOUND_MIXER_NRDEVICES];
    char *devicelabels[SOUND_MIXER_NRDEVICES]=SOUND_DEVICE_LABELS;

/*The next functions were taken from osdd-0.0.2,
written by W. Michael Petullo <osdd@flyn.org>,
along with some ideas, and slightly (one or two chars) modified by me*/

/* ============================ initialize_vols () ========================= */
void initialize_vols( int devmask)
{
    int i;
    int volume;
    for (i=0;i<SOUND_MIXER_NRDEVICES;i++) {
	if (devmask & (1 << i)) {
	    if (ioctl(dev, MIXER_READ(i), &volume) == -1) {
            	perror(mixer);
		mixerset.display=0;
	    }
	    old_vol[i] = vol[i] = volume;
	}
    }
}

/* ============================ display_channel () ========================= */
void display_channel(xosd * osd, int channel)
{
    char channel_name[BUFSIZ];
    if (!channel) 
	sprintf(channel_name, "Master Volume");
    else 
	sprintf(channel_name, "%s Volume", devicelabels[channel]);

    if (ioctl(dev, MIXER_READ(channel), &vol[channel]) == -1) {
        perror(mixer);
        mixerset.display = 0;
    }
    /*if muted, we say it!!*/
    if (!(vol[0] & 0xff || (vol[0] >> 8) & 0xff)) {
        xosd_display(osd, mixerset.yposition, XOSD_bottom, "Muted");
        xosd_display(osd, !mixerset.yposition, XOSD_bottom, "");
    }
    /* not muted, so lets check if the vol changed, if so, display
    them, if not, just ignore it*/
    else if (vol[channel] != old_vol[channel]) {
	xosd_display(osd, 0, XOSD_string, channel_name);
	xosd_display(osd, 1, XOSD_percentage,
		(((vol[channel]>>8)&0xff)+vol[channel]&0xff)/2);
    }
}
/* the next function is the main function of osdd-0.0.2, almost "as is" in the
original program, but i made it work as a thread, that is, i am making the whole
osdd-0.0.2 a thread of this program!!*/
/*================================= mixer_watch() ============================*/
void *mixer_watch(void *arg)
{
    int devmask;
    int i;
    /* we open the osd for the mixer*/
    /* Mixer usually goes on the bottom of the screen... so*/

    mixerset.yposition = XOSD_bottom;

    mixerosd = xosd_init(mixerset.font, mixerset.color, mixerset.delay,
		mixerset.xposition, mixerset.yposition,
		mixerset.soffset, mixerset.xoffset, mixerset.yoffset);

    /*open the mixer device for reading*/

    if ((dev = open(mixer, O_RDWR)) < 0) {
	perror("Error opening");
        perror(mixer);
        pthread_exit(0);
    }
    if (ioctl(dev, SOUND_MIXER_READ_DEVMASK, &devmask) == -1) {
	perror("Error reading");
        perror(mixer);
        pthread_exit(0);
    }
    /*initialize the volume.....*/
    initialize_vols(devmask);

    while(mixerset.display) {
	    for (i=0;i<SOUND_MIXER_NRDEVICES;i++) {
		if (devmask & (1 << i)) {
		    display_channel(mixerosd, i);
		    old_vol[i] = vol[i];
		}
	    }
	    usleep(1);
    }
    pthread_exit(0);
}

