#include "osdsh.h"
#include "../config.h"


/* ==============================global variables=============================*/
/* for the mixer stuff*/

char mixer[PATH_MAX +1] = MIXER;

/*basics*/

extern settings_t settings;
extern settings_t mixerset;
extern settings_t clockset;
extern settings_t pppset;
extern settings_t apmset;
extern xosd *mainosd;
extern xosd *mixerosd;
extern xosd *clockosd;
extern xosd *ppposd;
extern xosd *apmosd;
extern int clocksleep;
extern int apmsleep;

void *apm_watch(void *arg);
void *control_sh(void *arg);
void *mixer_watch(void *arg);
void *connection_watch(void *arg);
void *clock_display(void *arg);

/* ============================ usage () =================================== */
void usage(const int exitcode, const char *error, const char *more)
{
    fprintf(stderr, "osdsh %s\n", VERSION);
    fprintf(stderr, "usage: osdsh [options]\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "  -h        display usage\n");
    fprintf(stderr, "  -m mixer  set mixer         %s\n", MIXER);
    fprintf(stderr, "  -f font   set font          %s \n", FONT);
    fprintf(stderr, "  -c color  set color         %s \n", COLOR);
    fprintf(stderr, "  -d int    set OSD delay     %d \n", TIMEOUT);
    fprintf(stderr, "  -o int    set Shadow Offset %d \n", SHADOW_OFFSET);
    fprintf(stderr, "  -p <0|1>  position of the osd, 0 for bottom,\n");
    fprintf(stderr, "            1 for top. Default 1 (top).\n");
    fprintf(stderr, "\n");
    if (error)
	fprintf(stderr, "%s: %s\n\n", error, more);
    exit(exitcode);
}

/* ============================ set_defaults () ============================ */ 
void set_defaults(settings_t *settings)
{
    strncpy(settings->font, FONT, BUFSIZ);
    settings->font[BUFSIZ] = '\0';
    strncpy(settings->color, COLOR, BUFSIZ);
    settings->color[BUFSIZ] = '\0';
    settings->soffset = SHADOW_OFFSET;
    settings->yoffset = DEFAULT_OFFSET;
    settings->xoffset = DEFAULT_OFFSET;
    settings->xposition = OSD_XPOSITION;
    settings->yposition = OSD_YPOSITION;
    settings->delay = TIMEOUT;
    settings->display = 0;
}

/* ============================ parse_args () ============================== */
void parse_args(int argc, char *argv[], settings_t * settings)
{
    char c;
    int opt_index = 0;
    struct option opts[] = {
	{"help", 0, 0, 'h'},
	{"position", 1, 0, 'p'},
	{"soffset", 1, 0, 'o'},
	{"mixer", 1, 0, 'm'},
	{"font", 1, 0, 'f'},
	{"color", 1, 0, 'c'},
	{"delay", 1, 0, 'd'},
    };
    while ((c = getopt_long(argc, argv, "hm:f:c:p:d:o:", opts, &opt_index)) >= 0) {
	switch (c) {
	case 'h':
	    usage(0, NULL, NULL);
	    exit(0);
	case 'f':
	    strncpy(settings->font, optarg, BUFSIZ);
	    settings->font[BUFSIZ] = '\0';
	    break;
	case 'c':
	    strncpy(settings->color, optarg, BUFSIZ);
	    settings->color[BUFSIZ] = '\0';
	    break;
	case 'p':
		settings->yposition = atoi(optarg);
	    break;
	case 'd':
	    settings->delay = atoi(optarg);
	    break;
	case 'm':
	    strncpy(mixer, optarg, BUFSIZ);
	    mixer[BUFSIZ] = '\0';
	    break;
	case 'o':
	    settings->soffset = atoi(optarg);
	    break;
	default:
	    usage(1, NULL, NULL);
	    exit(1);
	}
    }
}

/* ============================ main () ==================================== */

int main(int argc, char *argv[], char *env[])
{
    pid_t childpid;

    set_defaults(&settings);

    parse_args(argc, argv, &settings);
    apmset = pppset = mixerset = clockset = settings;

    if((childpid=fork())<0) {
	perror("fork");
	exit(1);
    }
    if (childpid==0) {
	    control_sh(NULL);
    }
    return(0);
}
